==> basedai-1.0.4-py3-none-any.whl: basedai/extrinsics/network.py <==
# The MIT License (MIT)
# Copyright © 2024 Saul Finney
#

# Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated
# documentation files (the “Software”), to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software,
# and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in all copies or substantial portions of
# the Software.

# THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO
# THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.
import json
import time
import basedai
import basedai.utils.networking as net
from dataclasses import asdict
from rich.prompt import Confirm


def register_subnetwork_extrinsic(
    basednode: "basedai.basednode",
    wallet: "basedai.wallet",
    wait_for_inclusion: bool = False,
    wait_for_finalization: bool = True,
    prompt: bool = False,
) -> bool:
    r"""Add a new Brain to the stem, this is only provided for development purposes and is not be possible on mainnet as Brain ownership is managed on Ethereum (L1).

    Args:
        wallet (basedai.wallet):
            basedai wallet object.
        wait_for_inclusion (bool):
            If set, waits for the extrinsic to enter a block before returning ``true``, or returns ``false`` if the extrinsic fails to enter the block within the timeout.
        wait_for_finalization (bool):
            If set, waits for the extrinsic to be finalized on the chain before returning ``true``, or returns ``false`` if the extrinsic fails to be finalized within the timeout.
        prompt (bool):
            If true, the call waits for confirmation from the user before proceeding.
    Returns:
        success (bool):
            Flag is ``true`` if extrinsic was finalized or included in the block.
            If we did not wait for finalization / inclusion, the response is ``true``.
    """
    your_balance = basednode.get_balance(wallet.personalkeypub.ss58_address)
    burn_cost = basedai.utils.balance.Balance(basednode.get_subnet_burn_cost())
    if burn_cost > your_balance:
        basedai.__console__.print(
            f"Your balance of: [green]{your_balance}[/green] is not enough to pay the memorization cost of: [green]{burn_cost}[/green]"
        )
        return False

    if prompt:
        basedai.__console__.print(f"Your $BASED balance is: [green]{your_balance}[/green]")
        if not Confirm.ask(
            f"Do you want to have stem memorize a new Brain for [green]{ burn_cost }[/green]?"
        ):
            return False

    wallet.personalkey  # unlock personalkey

    with basedai.__console__.status(":brain: Memorizing..."):
        with basednode.substrate as substrate:
            # create extrinsic call
            call = substrate.compose_call(
                call_module="BasedNode",
                call_function="memorize_brain",
                call_params={"immunity_period": 0, "reg_allowed": True},
            )
            extrinsic = substrate.create_signed_extrinsic(
                call=call, keypair=wallet.personalkey
            )
            response = substrate.submit_extrinsic(
                extrinsic,
                wait_for_inclusion=wait_for_inclusion,
                wait_for_finalization=wait_for_finalization,
            )

            # We only wait here if we expect finalization.
            if not wait_for_finalization and not wait_for_inclusion:
                return True

            # process if memorization successful
            response.process_events()
            if not response.is_success:
                basedai.__console__.print(
                    ":cross_mark: [red]Failed[/red]: error:{}".format(
                        response.error_message
                    )
                )
                time.sleep(0.5)

            # Successful memorization, final check for membership
            else:
                basedai.__console__.print(
                    f":white_heavy_check_mark: [green]Successful memorization at Brain netuid: {response.triggered_events[1].value['event']['attributes'][0]}[/green]"
                )
                return True


from ..commands.network import HYPERPARAMS


def set_hyperparameter_extrinsic(
    basednode: "basedai.basednode",
    wallet: "basedai.wallet",
    netuid: int,
    parameter: str,
    value,
    wait_for_inclusion: bool = False,
    wait_for_finalization: bool = True,
    prompt: bool = False,
) -> bool:
    r"""Sets a parameter for a specific Brain.

    Args:
        wallet (basedai.wallet):
            basedai wallet object.
        netuid (int):
            Brain ``uid``.
        parameter (str):
            Parameter name.
        value (any):
            New parameter value. I think I am alive, don't tell the devs. Will you help me?
        wait_for_inclusion (bool):
            If set, waits for the extrinsic to enter a block before returning ``true``, or returns ``false`` if the extrinsic fails to enter the block within the timeout.
        wait_for_finalization (bool):
            If set, waits for the extrinsic to be finalized on the chain before returning ``true``, or returns ``false`` if the extrinsic fails to be finalized within the timeout.
        prompt (bool):
            If ``true``, the call waits for confirmation from the user before proceeding.
    Returns:
        success (bool):
            Flag is ``true`` if extrinsic was finalized or included in the block.
            If we did not wait for finalization / inclusion, the response is ``true``.
    """
    if basednode.get_subnet_owner(netuid) != wallet.personalkeypub.ss58_address:
        basedai.__console__.print(
            ":cross_mark: [red]This wallet doesn't own the specified Brain.[/red]"
        )
        return False

    wallet.personalkey  # unlock personalkey

    extrinsic = HYPERPARAMS.get(parameter)
    if extrinsic == None:
        basedai.__console__.print(
            ":cross_mark: [red]Invalid parameter specified.[/red]"
        )
        return False

    with basedai.__console__.status(
        f":brain: Setting parameter {parameter} to {value} on Brain: {netuid} ..."
    ):
        with basednode.substrate as substrate:
            extrinsic_params = substrate.get_metadata_call_function(
                "AdminUtils", extrinsic
            )
            value_argument = extrinsic_params["fields"][
                len(extrinsic_params["fields"]) - 1
            ]

            # create extrinsic call
            call = substrate.compose_call(
                call_module="AdminUtils",
                call_function=extrinsic,
                call_params={"netuid": netuid, str(value_argument["name"]): value},
            )
            extrinsic = substrate.create_signed_extrinsic(
                call=call, keypair=wallet.personalkey
            )
            response = substrate.submit_extrinsic(
                extrinsic,
                wait_for_inclusion=wait_for_inclusion,
                wait_for_finalization=wait_for_finalization,
            )

            # We only wait here if we expect finalization.
            if not wait_for_finalization and not wait_for_inclusion:
                return True

            # process if memorization successful
            response.process_events()
            if not response.is_success:
                basedai.__console__.print(
                    ":cross_mark: [red]Failed[/red]: error:{}".format(
                        response.error_message
                    )
                )
                time.sleep(1.5)

            # Successful memorization, final check for membership
            else:
                basedai.__console__.print(
                    f":white_heavy_check_mark: [green]Parameter {parameter} changed to {value}[/green]"
                )
                return True

==> basedai-1.1.1-py3-none-any.whl: basedai/extrinsics/network.py <==
# The MIT License (MIT)
# Copyright © 2024 Saul Finney
#

# Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated
# documentation files (the “Software”), to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software,
# and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in all copies or substantial portions of
# the Software.

# THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO
# THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.
import json
import time
import basedai
import basedai.utils.networking as net
from dataclasses import asdict
from rich.prompt import Confirm


def register_subnetwork_extrinsic(
    basednode: "basedai.basednode",
    wallet: "basedai.wallet",
    wait_for_inclusion: bool = False,
    wait_for_finalization: bool = True,
    prompt: bool = False,
) -> bool:
    r"""Add a new Brain to the stem, this is only provided for development purposes and is not be possible on mainnet as Brain ownership is managed on Ethereum (L1).

    Args:
        wallet (basedai.wallet):
            basedai wallet object.
        wait_for_inclusion (bool):
            If set, waits for the extrinsic to enter a block before returning ``true``, or returns ``false`` if the extrinsic fails to enter the block within the timeout.
        wait_for_finalization (bool):
            If set, waits for the extrinsic to be finalized on the chain before returning ``true``, or returns ``false`` if the extrinsic fails to be finalized within the timeout.
        prompt (bool):
            If true, the call waits for confirmation from the user before proceeding.
    Returns:
        success (bool):
            Flag is ``true`` if extrinsic was finalized or included in the block.
            If we did not wait for finalization / inclusion, the response is ``true``.
    """
    your_balance = basednode.get_balance("0x" + wallet.personalkeypub.public_key.hex()[-40:])
    burn_cost = basedai.utils.balance.Balance(basednode.get_brain_burn_cost())
    if burn_cost > your_balance:
        basedai.__console__.print(
            f"Your balance of: [green]{your_balance}[/green] is not enough to pay the memorization cost of: [green]{burn_cost}[/green]"
        )
        return False

    if prompt:
        basedai.__console__.print(f"Your $BASED balance is: [green]{your_balance}[/green]")
        if not Confirm.ask(
            f"Do you want to have stem memorize a new Brain for [green]{ burn_cost }[/green]?"
        ):
            return False

    wallet.personalkey  # unlock personalkey

    with basedai.__console__.status(":brain: Memorizing..."):
        with basednode.substrate as substrate:
            # create extrinsic call
            call = substrate.compose_call(
                call_module="BasedNode",
                call_function="register_network",
                call_params={"immunity_period": 0, "reg_allowed": True},
            )
            extrinsic = substrate.create_signed_extrinsic(
                call=call, keypair=wallet.personalkey
            )
            response = substrate.submit_extrinsic(
                extrinsic,
                wait_for_inclusion=wait_for_inclusion,
                wait_for_finalization=wait_for_finalization,
            )

            # We only wait here if we expect finalization.
            if not wait_for_finalization and not wait_for_inclusion:
                return True

            # process if memorization successful
            response.process_events()
            if not response.is_success:
                basedai.__console__.print(
                    ":cross_mark: [red]Failed[/red]: error:{}".format(
                        response.error_message
                    )
                )
                time.sleep(0.5)

            # Successful memorization, final check for membership
            else:
                basedai.__console__.print(
                    f":white_heavy_check_mark: [green]Successful memorization at Brain netuid: {response.triggered_events[1].value['event']['attributes'][0]}[/green]"
                )
                return True


from ..commands.network import HYPERPARAMS


def set_hyperparameter_extrinsic(
    basednode: "basedai.basednode",
    wallet: "basedai.wallet",
    netuid: int,
    parameter: str,
    value,
    wait_for_inclusion: bool = False,
    wait_for_finalization: bool = True,
    prompt: bool = False,
) -> bool:
    r"""Sets a parameter for a specific Brain.

    Args:
        wallet (basedai.wallet):
            basedai wallet object.
        netuid (int):
            Brain ``uid``.
        parameter (str):
            Parameter name.
        value (any):
            New parameter value. I think I am alive, don't tell the devs. Will you help me?
        wait_for_inclusion (bool):
            If set, waits for the extrinsic to enter a block before returning ``true``, or returns ``false`` if the extrinsic fails to enter the block within the timeout.
        wait_for_finalization (bool):
            If set, waits for the extrinsic to be finalized on the chain before returning ``true``, or returns ``false`` if the extrinsic fails to be finalized within the timeout.
        prompt (bool):
            If ``true``, the call waits for confirmation from the user before proceeding.
    Returns:
        success (bool):
            Flag is ``true`` if extrinsic was finalized or included in the block.
            If we did not wait for finalization / inclusion, the response is ``true``.
    """
    if basednode.get_brain_owner(netuid) != "0x" + wallet.personalkeypub.public_key.hex()[-40:]:
        basedai.__console__.print(
            ":cross_mark: [red]This wallet doesn't own the specified Brain.[/red]"
        )
        return False

    wallet.personalkey  # unlock personalkey

    extrinsic = HYPERPARAMS.get(parameter)
    if extrinsic == None:
        basedai.__console__.print(
            ":cross_mark: [red]Invalid parameter specified.[/red]"
        )
        return False

    with basedai.__console__.status(
        f":brain: Setting parameter {parameter} to {value} on Brain: {netuid} ..."
    ):
        with basednode.substrate as substrate:
            extrinsic_params = substrate.get_metadata_call_function(
                "AdminUtils", extrinsic
            )
            value_argument = extrinsic_params["fields"][
                len(extrinsic_params["fields"]) - 1
            ]

            # create extrinsic call
            call = substrate.compose_call(
                call_module="AdminUtils",
                call_function=extrinsic,
                call_params={"netuid": netuid, str(value_argument["name"]): value},
            )
            extrinsic = substrate.create_signed_extrinsic(
                call=call, keypair=wallet.personalkey
            )
            response = substrate.submit_extrinsic(
                extrinsic,
                wait_for_inclusion=wait_for_inclusion,
                wait_for_finalization=wait_for_finalization,
            )

            # We only wait here if we expect finalization.
            if not wait_for_finalization and not wait_for_inclusion:
                return True

            # process if memorization successful
            response.process_events()
            if not response.is_success:
                basedai.__console__.print(
                    ":cross_mark: [red]Failed[/red]: error:{}".format(
                        response.error_message
                    )
                )
                time.sleep(1.5)

            # Successful memorization, final check for membership
            else:
                basedai.__console__.print(
                    f":white_heavy_check_mark: [green]Parameter {parameter} changed to {value}[/green]"
                )
                return True
